//+------------------------------------------------------------------+
//|                                                  ClassExpert.mqh |
//|                                      Copyright vdv_2001 Software |
//|                                                 vdv_2001@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Dmitry Voronkov"
#property link      "vdv_2001@mail.ru"

#include <Arrays\List.mqh>
#include "ClassPriceHistogram.mqh"
#include "ClassProgressBar.mqh"
//+------------------------------------------------------------------+
//|    CExpert                                                  |
//|                                                    |
//+------------------------------------------------------------------+
class CExpert
  {
public:
   int               DaysForCalculation; //   (-1 ) / Days for calculation(-1 all)
   int               DayTheHistogram;    //    / Days The Histogram
   int               RangePercent;       //   / Range%
   color             InnerRange;         //    / Color internal range
   color             OuterRange;         //    / Color outer range
   color             ControlPoint;       //   (POC) / Color Point of Control
   bool              ShowValue;          //   / Show Value
                                         //    / Private variables of a class
private:
   CList             list_object;        //     CObject / The dynamic list of copies of class CObject 
   string            name_symbol;        //   / Symbol name
   int               count_bars;         //    / Quantity of day bars
   bool              event_on;           //    / Flag of processing of events

public:
   //   / The designer of a class
                     CExpert();
   //   / Class destructor
                    ~CExpert(){Deinit(REASON_CHARTCLOSE);}
   //   / Initialization method
   bool              Init();
   //   / Deinitialization method
   void              Deinit(const int reason);
   //   OnTick / Method of processing OnTick
   void              OnTick();
   //    OnChartEvent() / Method of processing of event OnChartEvent ()
   void              OnEvent(const int id,const long &lparam,const double &dparam,const string &sparam);
   //    OnTimer() / Method of processing of event OnTimer ()
   void              OnTimer();
  };
//+------------------------------------------------------------------+
//|     / The designer of a class                   |
//+------------------------------------------------------------------+
CExpert::CExpert()
  {
   name_symbol=NULL;
   count_bars=0;
   RangePercent=70;
   InnerRange=Indigo;
   OuterRange=Magenta;
   ControlPoint=Orange;
   ShowValue=true;
   DaysForCalculation=100;
   DayTheHistogram=10;
   event_on=false;
  }
//+------------------------------------------------------------------+
//|     / Initialization method                    |
//+------------------------------------------------------------------+
bool CExpert::Init()
  {
   int   rates_total,count;
   datetime day_time_open[];

//    / Check of change of the Symbol
   if(name_symbol==NULL || name_symbol!=Symbol())
     {
      list_object.Clear();
      event_on=false;   //     / We lower a flag of processing of events
      Sleep(100);
      name_symbol=Symbol();
      //      / Reception of a file of time of opening of days
      int err=0;
      do
        {
         //        / Account of quantity of days proceeding from an accessible history
         count_bars=Bars(NULL,PERIOD_D1);
         if(DaysForCalculation+1<count_bars)
            count=DaysForCalculation+1;
         else
            count=count_bars;
         if(DaysForCalculation<=0) count=count_bars;
         rates_total=CopyTime(NULL,PERIOD_D1,0,count,day_time_open);
         Sleep(1);
         err++;
        }
      while(rates_total<=0 && err<AMOUNT_OF_ATTEMPTS);
      if(err>=AMOUNT_OF_ATTEMPTS)
        {
         Print("There is no accessible history PERIOD_D1");
         name_symbol=NULL;
         return(false);
        }

      //   0    (   )
      // We check 0 day is current or not (for shares and Friday)
      if(day_time_open[rates_total-1]+PeriodSeconds(PERIOD_D1)>=TimeTradeServer())
         rates_total--;

      //        
      // We create object for a conclusion to the schedule of process of loading
      CProgressBar   *progress=new CProgressBar;
      progress.Create(0,"Loading",0,150,20);
      progress.Text("Calculation:");
      progress.Maximum=rates_total;
      //       CPriceHistogram       
      // In the given cycle there is a creation of object CPriceHistogram its initialization and entering into the list of objects
      for(int i=0;i<rates_total;i++)
        {
         CPriceHistogram  *hist_obj=new CPriceHistogram();
         //         hist_obj.StepHistigram(step);
         //      / We establish a flag of display of text labels
         hist_obj.ShowLevel(ShowValue);
         //   POCs / We establish colour POCs
         hist_obj.ColorPOCs(ControlPoint);
         //     / We establish colour in a range
         hist_obj.ColorInner(InnerRange);
         //     / We establish colour behind a range
         hist_obj.ColorOuter(OuterRange);
         //    / We establish range percent
         hist_obj.RangePercent(RangePercent);
         //         hist_obj.ShowSecondaryPOCs((i>=rates_total-DayTheHistogram),PeriodSeconds(PERIOD_D1));
         if(hist_obj.Init(day_time_open[i],day_time_open[i]+PeriodSeconds(PERIOD_D1),(i>=rates_total-DayTheHistogram)))
            list_object.Add(hist_obj);
         else
            delete hist_obj; //     / We delete if there was an error
         progress.Value(i);
        }
      delete progress;
      ChartRedraw(0);
      event_on=true;    //     / We hoist the colours processings of events
     }
   else
      OnTick();
//---
   return(true);
  }
//+------------------------------------------------------------------+
//|     OnTick / Method of processing OnTick           |
//+------------------------------------------------------------------+
void CExpert::OnTick(void)
  {
   int count;
//     / We check presence of new day
   if(count_bars!=Bars(NULL,PERIOD_D1))
     {
      name_symbol=NULL;
      Init();
     }
   else
     {
      count=list_object.Total();
      for(int i=0;i<count;i++)
        {
         CPriceHistogram  *hist_obj=list_object.GetNodeAtIndex(i);
         if(hist_obj.VirginPOCs())
            hist_obj.RefreshPOCs();
        }
     }
   return;
  }
//+------------------------------------------------------------------+
//|     / Deinitialization method                |
//+------------------------------------------------------------------+
void CExpert::Deinit(const int reason)
  {
   switch(reason)
     {
      case REASON_PARAMETERS: //    
         name_symbol=NULL;
         break;
      case REASON_ACCOUNT:    //  
      case REASON_CHARTCLOSE: // 
      case REASON_INITFAILED: // ,   OnInit()   
      case REASON_RECOMPILE:  // 
      case REASON_REMOVE:     //   
      case REASON_TEMPLATE:   //   
      case REASON_CHARTCHANGE://     
         break;
     }
   return;
  }
//+------------------------------------------------------------------------------------------+
//|      OnChartEvent() / Method of processing of event OnChartEvent () |
//+------------------------------------------------------------------------------------------+
void CExpert::OnEvent(const int id,const long &lparam,const double &dparam,const string &sparam)
  {
   if(event_on)   //        
     {            // If at present there is no class initialization
      switch(id)
        {
         case CHARTEVENT_KEYDOWN:
            //   ,      
            // Event of pressing of the keyboard when the schedule window is in focus
            break;
         case CHARTEVENT_OBJECT_CLICK:
            //      ,  
            // Event of click of the mouse on the graphic object, belonging to the schedule
            if(list_object.Total()!=0)
              {
               datetime serch=StringToTime(sparam);
               int count=list_object.Total();
               for(int i=0;i<count;i++)
                 {
                  CPriceHistogram  *hist_obj=list_object.GetNodeAtIndex(i);
                  if(hist_obj.GetStartDateTime()==serch)
                    {
                     hist_obj.ShowSecondaryPOCs(!hist_obj.ShowSecondaryPOCs());
                     color col=hist_obj.ColorInner();
                     hist_obj.ColorInner(hist_obj.ColorOuter());
                     hist_obj.ColorOuter(col);
                     ChartRedraw(0);
                     break;
                    }
                 }
              }
            break;
         case CHARTEVENT_OBJECT_DRAG:
            //       
            // Event of moving of graphic object by means of the mouse
            break;
         case CHARTEVENT_OBJECT_ENDEDIT:
            //          LabelEdit
            // Event of the termination of editing of the text in the field of input of graphic object LabelEdit
            break;
         default:
            //        :)
            // Here we will get if there will be that that brand new :)
            break;
        }
     }
  }
//+---------------------------------------------------------------------------------+
//|      OnTimer() / Method of processing of event OnTimer ()  |
//+---------------------------------------------------------------------------------+
void CExpert::OnTimer(void)
  {
   if(event_on)
     {
      //        :
      // EventSetTimer(  ); ,    : EventKillTimer();
      // Before use it is necessary to add in the designer of a class a line:
      // EventSetTimer(time in seconds); and in a destructor a line: EventKillTimer();
     }
  }
//+------------------------------------------------------------------+
